function [clm ier] = h3dformta(zk,rscale,center,nterms,src,charge)
% H3DFORMTA - col vec of local exp coeffs given sources (ie applies S2L)
% 
% Note the sources do not have the 1/4pi factor.
%
% [clm ier] = h3dformta(zk,rscale,center,nterms,src,charge)
%
%---------------------------------------------------------------------
%     INPUT:
%
%     zk         : Helmholtz coefficient (complex)
%     rscale     : real-valued scaling parameter used in forming expansion
%                                   (see h3dformmp1)
%     center(3)  : coordinates of the expansion center
%     nterms     : order of the j-expansion (aka p)
%     zsrc(3,ns) : array of source location vectors
%     charge(ns) : array of source charge strengths
%
%---------------------------------------------------------------------
%     OUTPUT:
%
%     clm (lpp-by-1) : column vector of local expansion coeffs
%                  where lpp = (nterms+1)^2
%     ier        : error code (see fortran for explanation)

ns = numel(charge);
lpp = (nterms+1)^2;  % # local exp coeffs

mex_id_ = 'h3dformtawrap(o int[x], i dcomplex[x], i double[x], i double[xx], i dcomplex[x], i int[x], i double[x], i int[x], o dcomplex[x])';
[ier, clm] = localexp3d(mex_id_, zk, rscale, src, charge, ns, center, nterms, 1, 1, 1, 3, ns, ns, 1, 3, 1, lpp);

if ier>0, error(sprintf('h3dformtawrap: ier = %d\n',ier)); end


% ========================== Source-2-local matrix ========================
