function [clm ier] = h3dformtadp(zk,rscale,center,nterms,src,dipstr,dipvec)
% H3DFORMTADP - col vec of local exp coeffs given dipoles (ie applies D2L)
% 
% Note the sources do not have the 1/4pi factor.
%
% [clm ier] = h3dformtadp(zk,rscale,center,nterms,src,dipstr,dipvec)
%
%---------------------------------------------------------------------
%     INPUT:
%
%     zk         : Helmholtz coefficient (complex)
%     rscale     : real-valued scaling parameter used in forming expansion
%                                   (see h3dformmp1)
%     center(3)  : coordinates of the expansion center
%     nterms     : order of the j-expansion (aka p)
%     zsrc(3,ns) : array of source location vectors
%     dipstr(ns) : array of dipole charge strengths
%     dipvec(3,ns) : array of dipole direction vectors
%
%---------------------------------------------------------------------
%     OUTPUT:
%
%     clm (lpp-by-1) : column vector of local expansion coeffs
%                  where lpp = (nterms+1)^2
%     ier        : error code (see fortran for explanation)

ns = numel(dipstr);
lpp = (nterms+1)^2;  % # local exp coeffs

mex_id_ = 'h3dformtadpwrap(o int[x], i dcomplex[x], i double[x], i double[xx], i dcomplex[x], i double[xx], i int[x], i double[x], i int[x], o dcomplex[x])';
[ier, clm] = localexp3d(mex_id_, zk, rscale, src, dipstr, dipvec, ns, center, nterms, 1, 1, 1, 3, ns, ns, 3, ns, 1, 3, 1, lpp);

if ier>0, error('h3dformtadpwrap: ier = %d\n',ier); end


% ========================== Dipole-2-local matrix ========================
