function [A ier] = h3dformtadpmatrix(zk,rscale,center,nterms,src,dipvec)
% H3DFORMTADPMATRIX - D2L matrix mapping dipoles to local exp coeffs
% 
% Note the sources do not have the 1/4pi factor.
%
% [A ier] = h3dformtadpmatrix(zk,rscale,center,nterms,src,dipvec)
%
%---------------------------------------------------------------------
%     INPUT:
%
%     zk         : Helmholtz coefficient (complex)
%     rscale     : real-valued scaling parameter used in forming expansion
%                                   (see h3dformmp1)
%     center(3)  : coordinates of the expansion center
%     nterms     : order of the j-expansion (aka p)
%     zsrc(3,ns) : array of source location vectors
%     dipvec(3,ns) : array of dipole direction vectors
%
%---------------------------------------------------------------------
%     OUTPUT:
%
%     A (lpp-by-ns) : matrix of D2L
%                  where lpp = (nterms+1)^2
%     ier        : error code (see fortran for explanation)

ns = size(src,2);
lpp = (nterms+1)^2;  % # local exp coeffs

mex_id_ = 'h3dformtadpmatrix(o int[x], i dcomplex[x], i double[x], i double[xx], i double[xx], i int[x], i double[x], i int[x], o dcomplex[xx])';
[ier, A] = localexp3d(mex_id_, zk, rscale, src, dipvec, ns, center, nterms, 1, 1, 1, 3, ns, 3, ns, 1, 3, 1, lpp, ns);

if ier>0, error('h3dformtadpmatrix: ier = %d\n',ier); end
