function [u un] = h3dtaeval(wavek,rscale,center,nterms,clm,ztarg,znor)
% H3DTAEVAL - evaluate local (j) expansion at targets (& n-derivs)
%
% u = h3dtaeval(wavek,rscale,center,nterms,clm,ztarg)
% [u un] = h3dtaeval(wavek,rscale,center,nterms,clm,ztarg,znor)
%
%     The j exp coeffs are ordered as a 1d array (1:(nterms+1)^2),
%     same as the column order in h3dtamatrix.
%
%     Adapted from fmmlib3d-1.2, and needs to link to those libraries.
%     Notes:
%     1) Ynm precomputation limited to nterms<=200, it seems; increase size
%     of wlege for more? (This is as in hfmm3dpart).
%
%     Alex Barnett 2/5/12
%
%---------------------------------------------------------------------
%     INPUT:
%
%     wavek      : the Helmholtz coefficient (complex)
%     rscale     : real-valued scaling parameter used in forming expansion
%                                   (see h3dformmp1)
%     center(3)  : coordinates of the expansion center
%     nterms     : order of the j-expansion
%     clm        : j-exp coeffs (complex)
%     ztarg(3,ntargs) : array of target location vectors
%     znor(3,ntargs)  : array of target normal vectors (optional;
%                  only used if un requested)
%
%---------------------------------------------------------------------
%     OUTPUT:
%
%     u(ntargs): col vec of potential at targets
%     un(ntargs): col vec of normal derivatives at targets (if requested)

ifnd = (nargout>1);
ntarg = size(ztarg,2);
if ifnd && (size(znor,2)~=ntarg)
   error('znor must have same # cols as ztarg!');
end
if nargin<7, znor = ztarg; end   % dummy

lpp = (nterms+1)^2;
ifnd = double(ifnd);
ntarg = double(ntarg);
nterms = double(nterms);

mex_id_ = 'h3dtaevalflat(i dcomplex[x], i double[x], i double[x], i int[x], i dcomplex[x], i int[x], i double[xx], i double[xx], i int[x], o int[x], o dcomplex[x], o dcomplex[x])';
[ier, u, un] = localexp3d(mex_id_, wavek, rscale, center, nterms, clm, ntarg, ztarg, znor, ifnd, 1, 1, 3, 1, lpp, 1, 3, ntarg, 3, ntarg, 1, 1, ntarg, ntarg);

if ier>0, error('h3dtaeval MEX func error: ier = %d\n',ier); end






% ========================== Source-2-local interface ========================
