function [A An] = h3dtamatrix(wavek,rscale,center,nterms,ztarg,znor)
% H3DTAMATRIX - fill dense local (j) expansion matrix at targets (& n-derivs)
%
% A = h3dtamatrix(wavek,rscale,center,nterms,ztarg)
% [A An] = h3dtamatrix(wavek,rscale,center,nterms,ztarg,znor)
%
%     Code to output dense matrix mapping local (j) expansion coeffs to
%     values (and possibly directional derivatives) at multiple targets
%     Each row of the matrix is filled by h3dtarowall_trunc.
%     One crucial aspect is that the j exp coeffs are reordered from
%     a 2d array (0:nterms,-nterms,nterms) to a 1d array (1:(nterms+1)^2),
%     giving the column order of the dense output matrices A and An.
%
%     Adapted from fmmlib3d-1.0, and needs to link to those libraries.
%     Notes:
%     1) Ynm precomputation limited to nterms<=200, it seems; increase size
%     of wlege for more? (This is as in hfmm3dpart).
%
%     Alex Barnett 3/28/12
%
%---------------------------------------------------------------------
%     INPUT:
%
%     wavek      : the Helmholtz coefficient (complex)
%     rscale     : real-valued scaling parameter used in forming expansion
%                                   (see h3dformmp1)
%     center(3)  : coordinates of the expansion center
%     nterms     : order of the j-expansion
%     ztarg(3,ntargs) : array of target location vectors
%     znor(3,ntargs)  : array of target normal vectors
%                  (only used if An requested)
%
%---------------------------------------------------------------------
%     OUTPUT:
%
%     A(ntargs,lpp): matrix of potential at targets (lpp = (nterms+1)^2)
%     An(ntargs,lpp): matrix of normal derivatives at targets (if requested)
%
%  See h3dtamatrix in h3dwrappers.f for ier error codes.

ifnd = (nargout>1);
ntarg = size(ztarg,2);
if ifnd && (size(znor,2)~=ntarg)
   error('znor must have same # cols as ztarg!');
end
if nargin<6, znor = ztarg; end   % dummy

lpp = (nterms+1)^2;
%A = zeros(ntarg,lpp)+1i*zeros(ntarg,lpp);
%An = 0;
%if ifnd,
%An = zeros(ntarg,lpp)+1i*zeros(ntarg,lpp);
%end

ifnd = double(ifnd);
ntarg = double(ntarg);
nterms = double(nterms);

mex_id_ = 'h3dtamatrix(i dcomplex[x], i double[x], i double[x], i int[x], i int[x], i double[xx], i double[xx], i int[x], o int[x], o dcomplex[xx], o dcomplex[xx])';
[ier, A, An] = localexp3d(mex_id_, wavek, rscale, center, nterms, ntarg, ztarg, znor, ifnd, 1, 1, 3, 1, 1, 3, ntarg, 3, ntarg, 1, 1, ntarg, lpp, ntarg, lpp);

if ier>0, error('h3dtamatrix MEX func error: ier = %d\n',ier); end



% ==========================================================================
