c     Driver test code for matrix of local expansion evaluations at multiple
c     targets in 3D Helmholtz.
c     Barnett 3/28/12
      program h3dtamatrix_dr
      implicit none
      integer n,m,nterms,nterms1,ntermsmax,ier,iffld,nlege,lw7,lused7
      integer ifpot,lpp,ntarg,t
c     choose the # targets here:
      parameter (ntermsmax = 200, ntarg=1000)
      real *8 pi,wlege(100000),rscale,ztarg(3,ntarg),znor(3,ntarg)
      real *8 t1,t2,center(3)
      complex *16 ima,wavek
      complex *16 pot(ntarg),fld(3,ntarg),nd
      complex *16, allocatable :: a(:,:), an(:,:), locexp(:,:)
      parameter (pi = 3.141592653589793d0, ima = (0, 1))
      call prini(6,0)

      wavek = 10
      do n=1,ntarg
         ztarg(1,n) = 0 + n/1000d0
         ztarg(2,n) = 0.3d0
         ztarg(3,n) = 0.6d0
         znor(1,n) = -1/3d0
c     the +n here makes normals not unit length, but still will work:
         znor(2,n) = 1/3d0 + n/1000d0
         znor(3,n) = 2/3d0
      enddo
      center(1) = 0
      center(2) = 0
      center(3) = 1
      rscale = 1

c     setup a j coeffs array
      nterms = 30
      allocate(locexp(0:nterms,-nterms:nterms))
      call h3dzero(locexp,nterms)
c     5th row in the a,an arrays...
      locexp(2,-2) = 1
      call prinf('nterms = *',nterms,1)
c     choose if to test direction derivative too...
      iffld = 1
      
c     test at the t'th target only (writing into 1st pot and fld elements)
      t = 3
      t1 = second()
      call h3dtaeval(wavek,rscale,center,locexp,nterms,
     1		ztarg(1,t),pot,iffld,fld,ier)
      nd = -(fld(1,1)*znor(1,t)+fld(2,1)*znor(2,t)+fld(3,1)*znor(3,t))
      t2 = second()
      call prinf('h3dtaeval done, ier = *',ier,1)
      call prin2('h3dtaeval done on t''th target, time = *',t2-t1,1)
c     remember values are complex:
      call prin2('pot = *',pot,2)
      if (iffld.eq.1) then
         call prin2('nd = *',nd,2)
      endif
c      call prin2('fld = *',fld,6)
            

c     Now try the fast Ynm, according to hfmm3dparttargmain in hfmm3dpart.f:
      nlege=ntermsmax
      lw7=100000
      call ylgndrfwini(nlege,wlege,lw7,lused7)
cc    write(*,*)' lused7 from  ylgndrfwini is',lused7
      
      t1 = second()
      ifpot = 1
c     zero the incoming arrays
      do n=1,ntarg
         pot(n) = 0
         fld(1,n) = 0
         fld(2,n) = 0
         fld(3,n) = 0
      enddo
c     Note this adds to existing numbers in pot and fld, doesn't overwrite!:
      call h3dtaevalall_trunc(wavek,rscale,center,locexp,nterms,nterms,
     1     ztarg,ntarg,ifpot,pot,iffld,fld,wlege,nlege,ier)
      nd = -(fld(1,t)*znor(1,t)+fld(2,t)*znor(2,t)+fld(3,t)*znor(3,t))
      t2 = second()
      call prinf('h3dtaevalall_trunc done, ier = *',ier,1)
      call prin2('h3dtaevalall_trunc done, time = *',t2-t1,1)
      call prin2('pot = *',pot(t),2)
      if (iffld.eq.1) then
         call prin2('nd = *',nd,2)
      endif
c     call prin2('fld = *',fld,6)


c     Now try our new routine
c     total number of expansion coefficients
      lpp = (nterms+1)**2
      write(*,*) ' number of matrix cols (j basis size) : ',lpp
      allocate(a(ntarg,lpp))
      if (iffld.eq.1) then
         allocate(an(ntarg,lpp))
      endif
      t1 = second()
      call h3dtamatrix(wavek,rscale,center,nterms,ntarg,ztarg,znor,
     1     iffld,ier,a,an)
      t2 = second()
      call prinf('h3dtamatrix done, ier = *',ier,1)
      call prin2('h3dtamatrix done, time = *',t2-t1,1)
c     2nd el is (1,-1):
      call prin2('pot = *',a(t,5),2)
      if (iffld.eq.1) then
         call prin2('nd = *',an(t,5),2)
      endif


      end
