      subroutine h3dtamatrix(wavek,rscale,center,nterms,ntargs,ztarg,
     1     znor,ifnd,ier,a,an)
C     Code to output dense matrix mapping local (j) expansion coeffs to
c     values (and possibly directional derivatives) at multiple targets
c     Each row of the matrix is filled by h3dtarowall_trunc.
c     One crucial aspect is that the j exp coeffs are reordered from
c     a 2d array (0:nterms,-nterms,nterms) to a 1d array (1:(nterms+1)^2),
c     giving the column order of the dense output matrices a and an.
c
c     Adapted from fmmlib3d-1.0, and needs to link to those libraries.
c     Notes:
c     1) Ynm precomputation limited to nterms<=200, it seems; increase size
c     of wlege for more? (This is as in hfmm3dpart).
c
c     Alex Barnett 3/28/12
c
c---------------------------------------------------------------------
c     INPUT:
c
c     wavek      : the Helmholtz coefficient
c     rscale     : real *8 scaling parameter used in forming expansion
c                                   (see h3dformmp1)
c     center(3)  : coordinates of the expansion center
c     nterms     : order of the j-expansion
c     ntargs     : number of targets (ie rows of output matrix)
c     ztarg(3,ntargs) : array of target location vectors
c     ifnd       : flag for normal derivative computation
c		                    ifnd=0  - is not computed
c		                    ifnd=1  - is computed
c     znor(3,ntargs)  : array of target normal vectors (only used if ifnd=1)
c
c---------------------------------------------------------------------
c     OUTPUT:
c
c     ier        : error return code
c		      ier=0	returned successfully
c		      ier=8 insuffficient workspace 
c		      ier=16 insufficient memory 
c                            in subroutine "jfuns3d"
c     a(ntargs,lpp): matrix of potential at targets (lpp = (nterms+1)^2)
c     an(ntargs,lpp): matrix of normal derivatives at targets (if requested)
c
c-----------------------------------------------------------------------
      implicit none
      integer nlege,lw7,lused7,ntargs,nterms,ifnd,ifpot,ier,jer,lpp,i
      real *8 wlege(100000),rscale,center(3),ztarg(3,ntargs)
      real *8 znor(3,ntargs)
      complex *16 wavek, a(ntargs,*),an(ntargs,*)
      complex *16, allocatable :: pots(:,:),nds(:,:), ur(:,:),
     1     utheta(:,:), uphi(:,:)
c     this is all stuff from the original _trunc allocation routine:
      real *8, allocatable :: w(:)
      integer lwfjs,iiscale,ipp,iephi,liscale,ifjs,lfjs,lfjder,ifjder
      integer ippd,lused,lephi

cc    write (*,*) wavek,rscale,center,nterms,ntargs,ztarg,
cc   1     znor,ifnd
cc    a(:,1:16)

c     Now try the fast Ynm, according to hfmm3dparttargmain in hfmm3dpart.f
      nlege=200
      lw7=100000
      call ylgndrfwini(nlege,wlege,lw7,lused7)
c     write(*,*)'h3dtamatrix: lused7 from ylgndrfwini is',lused7

c ... Assigning work spaces for various temporary arrays:
c     (taken from h3dtaeval_trunc, and added to
      ier=0
c
      lwfjs=nterms+1000
      ipp=1
      lpp=(nterms+1)**2+3
      ippd  = ipp+lpp
c
      iephi=ippd+lpp
      lephi=2*(2*nterms+1)+7
c
      iiscale=iephi+lephi
      liscale=(lwfjs+1)+3
c
      ifjs=iiscale+liscale
      lfjs=2*(lwfjs+1)+3
c
      ifjder=ifjs+lfjs
      lfjder=2*(nterms+1)+3
c
      lused=ifjder+lfjder
      allocate(w(lused))

c     alex's extra allocations... all of size of a set of j exp coeffs
      allocate(ur(0:nterms,-nterms:nterms))
      allocate(utheta(0:nterms,-nterms:nterms))
      allocate(uphi(0:nterms,-nterms:nterms))
      allocate(pots(0:nterms,-nterms:nterms))
      allocate(nds(0:nterms,-nterms:nterms))

c     loop over targets and write a row of a (and an) matrices for each one
      do i=1,ntargs
         call h3dtarow_trunc0(jer,wavek,rscale,center,nterms,
     1        nterms,ztarg(1,i),znor(1,i),pots,ifnd,nds,w(ipp),w(ippd),
     1        w(iephi),w(ifjs),w(ifjder),lwfjs,w(iiscale),
     1        ur,utheta,uphi,wlege,nlege)
c     the following memcpy type call is most (70-95%) of the CPU time! yuk
C     It's the large stride of RAM access that screws it up.
         call flattenlocexpz(nterms,pots,a(i,1),ntargs)
         if (ifnd.eq.1) then
            call flattenlocexpz(nterms,nds,an(i,1),ntargs)
         endif         
      enddo
      if (jer.ne.0) ier=jer
      end


c**********************************************************************
      subroutine flattenlocexpz(nterms,in,out,stride)
c     unpacks a complex *16 2d local expansion array (in) into a 1d list (out).
c     Output array is written to using given stride.
c     Alex Barnett 3/28/12
      implicit none
      integer i,n,m,nterms,stride
      complex *16 in(0:nterms,-nterms:nterms), out(*)
      
      i = 1
      do n=0,nterms
         do m=-n,n
            out(i) = in(n,m)
            i = i+stride
         enddo
      enddo
      end

c**********************************************************************
      subroutine stacklocexpz(nterms,in,out,stride)
c     packs a complex *16 1d local expansion list (in) into a 2d array (out).
c     Input array is read using given stride.
c     Alex Barnett 2/4/13
      implicit none
      integer i,n,m,nterms,stride
      complex *16 out(0:nterms,-nterms:nterms), in((nterms+1)**2)
      
      i = 1
      do n=0,nterms
         do m=-n,n
            out(n,m) = in(i)
            i = i+stride
         enddo
      enddo
      end


c**********************************************************************
      subroutine h3dtaevalflat(wavek,rscale,center,nterms,clm,
     $     ntargs,ztarg,znor,ifnd,ier,u,un)
c**********************************************************************
c
c     This subroutine evaluates a j-expansion centered at CENTER
c     at the target points ztarg, and if ifnd true, its normal derivs.
c
c     pot =  sum sum  locexp(n,m) j_n(k r) Y_nm(theta,phi)
c             n   m
c
c     The locexp (clm) is passed in as a 1d list, ie h3dtamatrix format
c     Alex Barnett 2/5/12.
c     OpenMP version 8/6/14
c---------------------------------------------------------------------
c     INPUT:
c
c     wavek      : the Helmholtz coefficient
c     rscale     : scaling parameter used in forming expansion
c                                   (see h3dformmp1)
c     center(3)  : coordinates of the expansion center
c     nterms     : order of the j-expansion
c     clm(lpp)   : coeffs of the j-expansion
c     ntargs     : number of targets
c     ztarg(3,ntargs)   : target coords vector
c     znor(3,ntargs)    : target normals vector
c     ifnd       : flag for normal-deriv computation
c		        ifnd=0  - is not computed
c		        ifnd=1  - is computed
c---------------------------------------------------------------------
c     OUTPUT:
c
c     ier          : error return code
c		      ier=0	returned successfully
c		      ier=8 insuffficient workspace 
c		      ier=16 insufficient memory 
c                            in subroutine "jfuns3d"
c     u(ntargs)    : potentials at ztarg
c     un(ntargs)   : normal-derivs at ztarg (if requested)
c
c     NOTE: Parameter lwfjs is set to nterms+1000
c           Should be sufficient for any Helmholtz parameter
c---------------------------------------------------------------------
      implicit none
      integer nlege,lw7,lused7,ntargs,nterms,ifnd,ier,jer,lpp,i
      real *8 wlege(100000),rscale,center(3),ztarg(3,ntargs)
      real *8 znor(3,ntargs)
      complex *16 wavek, u(ntargs),un(ntargs),pot0,fld0(3)
      complex *16 locexp(0:nterms,-nterms:nterms), clm((nterms+1)**2)
c     this is all stuff from the original _trunc allocation routine:
      real *8, allocatable :: w(:)
      integer lwfjs,iiscale,ipp,iephi,liscale,ifjs,lfjs,lfjder,ifjder
      integer ippd,lused,lephi

c     Following setup taken from h3dtamatrix....
c     Now try the fast Ynm, according to hfmm3dparttargmain in hfmm3dpart.f
      nlege=200
      lw7=100000
      call ylgndrfwini(nlege,wlege,lw7,lused7)
c     write(*,*)'h3dtamatrix: lused7 from ylgndrfwini is',lused7

c ... Assigning work spaces for various temporary arrays:
c     (taken from h3dtaeval_trunc, and added to
      ier=0
c
      lwfjs=nterms+1000
      ipp=1
      lpp=(nterms+1)**2+3
      ippd  = ipp+lpp
c
      iephi=ippd+lpp
      lephi=2*(2*nterms+1)+7
c
      iiscale=iephi+lephi
      liscale=(lwfjs+1)+3
c
      ifjs=iiscale+liscale
      lfjs=2*(lwfjs+1)+3
c
      ifjder=ifjs+lfjs
      lfjder=2*(nterms+1)+3
c
      lused=ifjder+lfjder
      allocate(w(lused))

c     convert to fmmlib locexp format...
c      allocate(locexp(0:nterms,-nterms:nterms))
      call stacklocexpz(nterms,clm,locexp,1)

c     loop over targets code, tweaked from FMMLIB's h3dtaevalall_trunc...
C$OMP PARALLEL DO
c     following SHARED's not needed. What's crucial is the PRIVATE w (workspace)
CCC$OMP$SHARED(wavek,nterms,ntargs,rscale,locexp,ifjder,znor)
CCC$OMP$SHARED(u,un,center)
CCC$OMP$SHARED(wlege,nlege,iiscale,lwfjs,w,ifnd,ifjs,iephi,ippd,ipp)
c     lastprivate here keeps only the last copy of jer from a thread...
C$OMP$PRIVATE (pot0,fld0,w) LASTPRIVATE(jer)
c     uncomment following to test non-parallel version...
CCC$OMP$NUM_THREADS(1)
      do i=1,ntargs
c     writes into u(i), and fld0 if needed
         call h3dtaeval_trunc0(jer,wavek,rscale,center,locexp,
     $        nterms,nterms,ztarg(1,i),
     1        pot0,ifnd,fld0,w(ipp),w(ippd),w(iephi),w(ifjs),
     2        w(ifjder),lwfjs,w(iiscale),wlege,nlege)
         u(i) = pot0
c     note minus sign here to get normal deriv
         if( ifnd .eq. 1 ) un(i) = -(fld0(1)*znor(1,i) +
     $        fld0(2)*znor(2,i) + fld0(3)*znor(3,i))
      enddo
C$OMP END PARALLEL DO
      if (jer.ne.0) ier=16

      return
      end



c**********************************************************************
      subroutine h3dtarow_trunc0(ier,wavek,rscale,center,
     $     nterms,nterms1,ztarg,znor,
     $     pots,iffld,nds,pp,ppd,ephi,fjs,fjder,lwfjs,iscale,
     $     ur,utheta,uphi,wlege,nlege)
c**********************************************************************
c
c     This is adapted from fmmlib3d-1.0 routine h3dtaeval_trunc0. The
c     only differences are that locexp is not passed in; instead the coeffs
c     array (of size locexp) that would have been dotted into this is returned
c     in pots, and the same size coeffs array for the normal-deriv of pot in
c     the direction znor is returned in nds (if iffld=1, as usual).
c     Note: nd = znor dot grad pot.
c     So, there is no minus sign as in fld = -grad pot.
c
c     NEW INPUTS:
c     znor(3)    : target unit normal vector for computation of normal deriv
c     ur(0:nterms,-nterms:nterms) : workspace for this routine (must be
c                                   allocated already)
c     utheta(0:nterms,-nterms:nterms) : workspace
c     uphi(0:nterms,-nterms:nterms) : workspace
c
c     NEW OUTPUTS:
c     pots(0:nterms,-nterms:nterms) : array of potentials for each j basis func
c                  ordered as in locexp in h3dtaeval.
c     nds(0:nterms,-nterms:nterms) : array of potentials for each j basis func
c
c     Alex Barnett 3/28/12
c
c----------------------------------------------------------------------
      implicit real *8 (a-h,o-z)
      integer iscale(0:1)
      real *8 center(3),ztarg(3),znor(3),zdiff(3)
      real *8 pp(0:nterms,0:nterms)
      real *8 ppd(0:nterms,0:nterms)
      complex *16 wavek,ephi1,ephi1inv,pots(0:nterms,-nterms:nterms)
      complex *16 nds(0:nterms,-nterms:nterms)
      complex *16 ur(0:nterms,-nterms:nterms)
      complex *16 uphi(0:nterms,-nterms:nterms)
      complex *16 utheta(0:nterms,-nterms:nterms)
      complex *16 ephi(-nterms-1:nterms+1)
      complex *16 fjsuse,fjs(0:1),fjder(0:1)
c
      complex *16 eye
      complex *16 ztmp,z
      complex *16 ztmp1,ztmp2,ztmp3,ztmpsum
      complex *16 ux,uy,uz
c
      data eye/(0.0d0,1.0d0)/
c
      ier=0
      done=1.0d0
c
      zdiff(1)=ztarg(1)-center(1)
      zdiff(2)=ztarg(2)-center(2)
      zdiff(3)=ztarg(3)-center(3)
c
c     Convert to spherical coordinates
c
      call cart2polar(zdiff,r,theta,phi)
      ctheta = dcos(theta)
      stheta=sqrt(done-ctheta*ctheta)
      cphi = dcos(phi)
      sphi = dsin(phi)
      ephi1 = dcmplx(cphi,sphi)
c
c     compute e^{eye*m*phi} array.
c
c
      ephi(0)=1.0d0
      ephi(1)=ephi1
      ephi(-1)=dconjg(ephi1)
      do i=2,nterms+1
         ephi(i)=ephi(i-1)*ephi1
         ephi(-i)=ephi(-i+1)*ephi(-1)
      enddo
c
c     compute coefficients in change of variables from spherical
c     to Cartesian gradients. In phix, phiy, we leave out the 
c     1/sin(theta) contribution, since we use values of Ynm (which
c     multiplies phix and phiy) that are scaled by 
c     1/sin(theta).
c
c     In thetax, thetaty, phix, phiy we leave out the 1/r factors in the 
c     change of variables to avoid blow-up at the origin.
c     For the n=0 mode, it is not relevant. For n>0 modes,
c     we use the recurrence relation 
c
c     (2n+1)fjs_n(kr)/(kr) = fjs(n+1)*rscale + fjs(n-1)/rscale
c
c     to avoid division by r. The variable fjsuse is set to fjs(n)/r:
c
c           fjsuse = fjs(n+1)*rscale + fjs(n-1)/rscale
c	    fjsuse = wavek*fjsuse/(2*n+1.0d0)
c
c     
c
      if (iffld.eq.1) then
         rx = stheta*cphi
ccc         thetax = ctheta*cphi/r
ccc         phix = -sphi/r
         thetax = ctheta*cphi
         phix = -sphi
         ry = stheta*sphi
ccc         thetay = ctheta*sphi/r
ccc         phiy = cphi/r
         thetay = ctheta*sphi
         phiy = cphi
         rz = ctheta
ccc         thetaz = -stheta/r
         thetaz = -stheta
         phiz = 0.0d0
      endif
c
c     get the associated Legendre functions:
c
      if (iffld.eq.1) then
ccc         call ylgndr2s(nterms,ctheta,pp,ppd)
         call ylgndr2sfw(nterms1,ctheta,pp,ppd,wlege,nlege)
      else
ccc         call ylgndr(nterms,ctheta,pp)
         call ylgndrfw(nterms1,ctheta,pp,wlege,nlege)
      endif
c
c     get the spherical Bessel functions and their derivatives.
c
      ifder=iffld
      z=wavek*r
cc      write (*,*)jer,nterms1,z,rscale,fjs,ifder,fjder,
cc     1	      lwfjs,iscale,ntop
      call jfuns3d(jer,nterms1,z,rscale,fjs,ifder,fjder,
     1	      lwfjs,iscale,ntop)
      if (jer.ne.0) then
         ier=8
         return
      endif
c
c     scale derivatives of Bessel functions so that they are
c     derivatives with respect to r.
c
c     seems to do n=m=0 only:
      pots(0,0)=fjs(0)
      if (iffld.eq.1) then
         do i=0,nterms1
            fjder(i)=fjder(i)*wavek
         enddo
         ur(0,0) = fjder(0)
         utheta(0,0) = 0.0d0
         uphi(0,0) = 0.0d0
      endif
c
c     compute the potential and the field: (n>0, first m=0 then |m|>0)
c
      if (iffld.eq.1) then
         do n=1,nterms1
            pots(n,0) = fjs(n)*pp(n,0)
	    ur(n,0) = fjder(n)*pp(n,0)
	    fjsuse = fjs(n+1)*rscale + fjs(n-1)/rscale
	    fjsuse = wavek*fjsuse/(2*n+1.0d0)
	    utheta(n,0) = -fjsuse*ppd(n,0)*stheta
            uphi(n,0) = 0d0
	    do m=1,n
	       ztmp1=fjs(n)*pp(n,m)*stheta
               pots(n,m) = ztmp1*ephi(m)
               pots(n,-m) = ztmp1*ephi(-m)
c     note my use of ztmp2 is different from Zydrunas+Leslie here:
               ztmp2 = fjder(n)*pp(n,m)*stheta
               ur(n,m) = ephi(m)*ztmp2
               ur(n,-m) = ephi(-m)*ztmp2
	       utheta(n,m) = -ephi(m)*fjsuse*ppd(n,m)
	       utheta(n,-m) = -ephi(-m)*fjsuse*ppd(n,m)
               ztmp2 = eye*m*fjsuse*pp(n,m)
               uphi(n,m) = ztmp2*ephi(m)
               uphi(n,-m) = -ztmp2*ephi(-m)
c     could be improved, by using half the space; who cares
            enddo
         enddo
ccc	 call prin2(' ur is *',ur,2)
ccc	 call prin2(' utheta is *',utheta,2)
ccc	 call prin2(' uphi is *',uphi,2)

c     create nds from znor dot grad u.......
         do n=0,nterms1
            do m=-n,n
ccc            write(*,*)' n,m=',n,m
               ux = ur(n,m)*rx + utheta(n,m)*thetax + uphi(n,m)*phix
               uy = ur(n,m)*ry + utheta(n,m)*thetay + uphi(n,m)*phiy
               uz = ur(n,m)*rz + utheta(n,m)*thetaz + uphi(n,m)*phiz
               nds(n,m) = ux*znor(1) + uy*znor(2) + uz*znor(3) 
            enddo
         enddo
      else
c     pot only...
         do n=1,nterms1
	    pots(n,0) = fjs(n)*pp(n,0)
	    do m=1,n
	       ztmp1=fjs(n)*pp(n,m)
               pots(n,m) = ztmp1*ephi(m)
               pots(n,-m) = ztmp1*ephi(-m)
            enddo
         enddo
      endif
      return
      end
c


c**********************************************************************
      subroutine h3dformtawrap
     $     (ier,zk,rscale,sources,charge,ns,center,
     $     nterms,clm)
c**********************************************************************
c
c     Largely taken from h3dformta_trunc from FMMLIB3D 1.2, Barnett 1/6/13
c
c     This subroutine creates a local (j) expansion about the point
c     CENTER due to the NS sources at the locations SOURCES(3,*).
c     This is the memory management routine. Work is done in the
c     secondary call to h3dformta1/h3dformta0.  (& seemingly their trunc
c     versions...)
c
c     note routine name can't have _ (underscore) in it for Mwrap to work
c----------------------------------------------------------------------
c     INPUT:
c
c     zk       : Helmholtz coefficient
c     rscale   : scaling parameter
c                     should be less than one in magnitude.
c                     Needed for low frequency regime only
c                     with rsclale abs(wavek) recommended.
c     sources   : coordinates of the sources
c     charge    : charge strengths
c     ns        : number of sources
c     center    : coordinates of the expansion center
c     nterms    : order of the j-expansion
c----------------------------------------------------------------------
c     OUTPUT:
c
c     ier       : error return code
c		  ier=0	returned successfully;
c	 	  ier=4  d is out of range in h3dall
c
c     clm    : vec of coeffs for the j-expansion, in h3dtamatrix order
c----------------------------------------------------------------------

      implicit none
      integer ier,ns,nterms,i,l,m,nlege,lw7,lused7
      real *8 rscale,wlege(100000),sources(3,ns),center(3)
      complex *16 zk,locexp(0:nterms,-nterms:nterms), charge(ns)
      complex *16 eye, clm((nterms+1)**2)
      data eye/(0.0d0,1.0d0)/

c     Setup for fast Ynm, according to hfmm3dparttargmain in hfmm3dpart.f
      nlege=200
      lw7=100000
      call ylgndrfwini(nlege,wlege,lw7,lused7)
c     write(*,*)'h3dformta_wrap: lused7 from ylgndrfwini is',lused7

c     initialize local exp
      do l = 0,nterms
         do m = -l,l
            locexp(l,m) = 0.0d0
         enddo
      enddo

C$OMP PARALLEL DO REDUCTION(+:locexp)
C$OMP$SHARED(zk,nterms,ns,rscale,sources,center,charge,wlege,nlege)
c     the lastprivate here keeps only the last copy of ier from a thread...
C$OMP$PRIVATE(i) LASTPRIVATE(ier)
CCC$OMP$NUM_THREADS(1)
c     following loops over srcs, adding in effect to locexp...
      do i = 1,ns
         call h3dformta_trunc1(ier,zk,rscale,sources(1,i),charge(i),
     1		center,nterms,nterms,locexp,wlege,nlege)
      enddo
C$OMP END PARALLEL DO
c     (I set nterms1=nterms)
c
c     scale by (i*k)
      do l = 0,nterms
         do m=-l,l
            locexp(l,m) = locexp(l,m)*eye*zk
         enddo
      enddo
C
c     reorder to a single vector clm, same ordering as h3dtamatrix above...
      call flattenlocexpz(nterms,locexp,clm,1)
      return
      end
c

c**********************************************************************
      subroutine h3dformtamatrix
     $     (ier,zk,rscale,sources,ns,center,
     $     nterms,A)
c**********************************************************************
c
c     Returns S2L matrix, ie matrix used in h3dformtawrap
c     Barnett 1/16/13
c
c     This subroutine creates a local (j) expansion about the point
c     CENTER due to the NS sources at the locations SOURCES(3,*).
c     This is the memory management routine. Work is done in the
c     secondary call to h3dformta1/h3dformta0.  (& seemingly their trunc
c     versions...)
c
c     note routine name can't have _ (underscore) in it for Mwrap to work
c----------------------------------------------------------------------
c     INPUT:
c
c     zk       : Helmholtz coefficient
c     rscale   : scaling parameter
c                     should be less than one in magnitude.
c                     Needed for low frequency regime only
c                     with rsclale abs(wavek) recommended.
c     sources   : coordinates of the sources
c     ns        : number of sources
c     center    : coordinates of the expansion center
c     nterms    : order of the j-expansion
c----------------------------------------------------------------------
c     OUTPUT:
c
c     ier       : error return code
c		  ier=0	returned successfully;
c	 	  ier=4  d is out of range in h3dall
c
c     A    : for the j-expansion, each col in h3dtamatrix order
c----------------------------------------------------------------------

      implicit none
      integer ier,ns,nterms,i,j,l,m,nlege,lw7,lused7
      real *8 rscale,wlege(100000),sources(3,ns),center(3)
      complex *16 zk,locexp(0:nterms,-nterms:nterms)
      complex *16 eye, A((nterms+1)**2,ns),dummy
      data eye/(0.0d0,1.0d0)/

c     Setup for fast Ynm, according to hfmm3dparttargmain in hfmm3dpart.f
      nlege=200
      lw7=100000
      call ylgndrfwini(nlege,wlege,lw7,lused7)
c     write(*,*)'h3dformta_wrap: lused7 from ylgndrfwini is',lused7

c     dummy is a unit charge strength, scaled by 1*k needed
      dummy = eye*zk

C$OMP PARALLEL DO SHARED(A,zk,nterms,ns,rscale,sources,dummy,center)
c     the lastprivate here keeps only the last copy of ier from a thread...
C$OMP$SHARED(wlege,nlege) PRIVATE(i,j,l,m,locexp) LASTPRIVATE(ier)
CCC$OMP$NUM_THREADS(1)
c     following loops over srcs, getting locexp then writing as col of A...
      do i = 1,ns
         do l = 0,nterms
            do m=-l,l
               locexp(l,m) = 0.0d0
            enddo
         enddo
         call h3dformta_trunc1(ier,zk,rscale,sources(1,i),dummy,
     1		center,nterms,nterms,locexp,wlege,nlege)
c     (I set nterms1=nterms)
c     reorder to a single col of A, same order as h3dtamatrix above
c     Seems to be 10% faster than flattenlocexpz(nterms,locexp,A(1,i),1)
         j = 1
         do l=0,nterms
            do m=-l,l
               A(j,i) = locexp(l,m)
               j = j+1
            enddo
         enddo
      enddo
C$OMP END PARALLEL DO

      return
      end
c


c**********************************************************************
      subroutine h3dformtadpwrap
     $     (ier,zk,rscale,sources,dipstr,dipvec,ns,center,
     $     nterms,clm)
c**********************************************************************
c
c     Largely taken from h3dformta_dp_trunc from FMMLIB3D 1.2,
c     and changed only slightly from h3dformtawrap above. Barnett 1/15/13
c
c     This subroutine creates a local (j) expansion about the point
c     CENTER due to the NS dipoles at the locations SOURCES(3,*).
c     This is the memory management routine. Work is done in the
c     secondary call to h3dformta1/h3dformta0.  (& seemingly their trunc
c     versions...)
c
c     note routine name can't have _ (underscore) in it for Mwrap to work
c----------------------------------------------------------------------
c     INPUT:
c
c     zk       : Helmholtz coefficient
c     rscale   : scaling parameter
c                     should be less than one in magnitude.
c                     Needed for low frequency regime only
c                     with rsclale abs(wavek) recommended.
c     sources   : coordinates of the sources
c     dipstr    : dipole strengths
c     dipvec    : dipole direction vectors
c     ns        : number of sources
c     center    : coordinates of the expansion center
c     nterms    : order of the j-expansion
c----------------------------------------------------------------------
c     OUTPUT:
c
c     ier       : error return code
c		  ier=0	returned successfully;
c	 	  ier=4  d is out of range in h3dall
c
c     clm    : vec of coeffs for the j-expansion, in h3dtamatrix order
c----------------------------------------------------------------------

      implicit none
      integer ier,ns,nterms,i,l,m,nlege,lw7,lused7
      real *8 rscale,wlege(100000),sources(3,ns),center(3),dipvec(3,ns)
      complex *16 zk,locexp(0:nterms,-nterms:nterms)
      complex *16 eye, clm((nterms+1)**2),dipstr(ns)
      data eye/(0.0d0,1.0d0)/

c     Setup for fast Ynm, according to hfmm3dparttargmain in hfmm3dpart.f
      nlege=200
      lw7=100000
      call ylgndrfwini(nlege,wlege,lw7,lused7)
c     write(*,*)'h3dformta_wrap: lused7 from ylgndrfwini is',lused7

c     initialize local exp
      do l = 0,nterms
         do m = -l,l
            locexp(l,m) = 0.0d0
         enddo
      enddo

C$OMP PARALLEL DO REDUCTION(+:locexp)
C$OMP$SHARED(zk,nterms,ns,rscale,sources,center)
c     the lastprivate here keeps only the last copy of ier from a thread...
C$OMP$SHARED(dipstr,dipvec,wlege,nlege) PRIVATE(i) LASTPRIVATE(ier)
CCC$OMP$NUM_THREADS(1)
c     following loops over srcs, adding in effect to locexp...
      do i = 1,ns
         call h3dformta1_dp_trunc(ier,zk,rscale,sources(1,i),dipstr(i),
     1		dipvec(1,i),center,nterms,nterms,locexp,wlege,nlege)
      enddo
C$OMP END PARALLEL DO
c     (I set nterms1=nterms)
c
c     scale by (i*k)
      do l = 0,nterms
         do m=-l,l
            locexp(l,m) = locexp(l,m)*eye*zk
         enddo
      enddo
C
c     reorder to a single vector clm, same ordering as h3dtamatrix above...
      call flattenlocexpz(nterms,locexp,clm,1)
      return
      end
c


c**********************************************************************
      subroutine h3dformtadpmatrix
     $     (ier,zk,rscale,sources,dipvec,ns,center,
     $     nterms,A)
c**********************************************************************
c
c     Returns dense matrix D2L, ie matrix from h3dformtadpwrap
c     Barnett 1/16/13
c
c----------------------------------------------------------------------
c     INPUT:
c
c     zk       : Helmholtz coefficient
c     rscale   : scaling parameter
c                     should be less than one in magnitude.
c                     Needed for low frequency regime only
c                     with rsclale abs(wavek) recommended.
c     sources   : coordinates of the sources
c     dipvec    : dipole direction vectors
c     ns        : number of sources
c     center    : coordinates of the expansion center
c     nterms    : order of the j-expansion
c----------------------------------------------------------------------
c     OUTPUT:
c
c     ier       : error return code
c		  ier=0	returned successfully;
c	 	  ier=4  d is out of range in h3dall
c
c     A    : D2L matrix, size lpp*ns, each col in h2tamatrix order
c----------------------------------------------------------------------

      implicit none
      integer ier,ns,nterms,i,j,l,m,nlege,lw7,lused7
      real *8 rscale,wlege(100000),sources(3,ns),center(3),dipvec(3,ns)
      complex *16 zk,locexp(0:nterms,-nterms:nterms)
      complex *16 eye, A((nterms+1)**2,ns),dummy
      data eye/(0.0d0,1.0d0)/

c     Setup for fast Ynm, according to hfmm3dparttargmain in hfmm3dpart.f
      nlege=200
      lw7=100000
      call ylgndrfwini(nlege,wlege,lw7,lused7)
c     write(*,*)'h3dformta_wrap: lused7 from ylgndrfwini is',lused7

c      call prini(6,13)
c      call prin2('zk=*',zk,2)

c     dummy is a unit charge strength scaled by needed i*k
      dummy = eye*zk

C$OMP PARALLEL DO SHARED(A,zk,nterms,ns,rscale,sources,dummy,dipvec)
C$OMP$SHARED(center,wlege,nlege)
c     lastprivate here keeps only the last copy of ier from a thread...
C$OMP$PRIVATE(i,j,l,m,locexp) LASTPRIVATE(ier)
CCC$OMP$NUM_THREADS(1)
c     following loops over dipoles, getting locexp then writing as col of A...
      do i = 1,ns
c     initialize local exp
         do l = 0,nterms
            do m = -l,l
               locexp(l,m) = 0.0d0
            enddo
         enddo
         call h3dformta1_dp_trunc(ier,zk,rscale,sources(1,i),dummy,
     1        dipvec(1,i),center,nterms,nterms,locexp,wlege,nlege)
c     (I set nterms1=nterms)

c     reorder to a single col of A, same ordering as h3dtamatrix above...
c     Seems to be 10% faster than flattenlocexpz(nterms,locexp,A(1,i),1)
         j = 1
         do l=0,nterms
            do m=-l,l
               A(j,i) = locexp(l,m)
               j = j+1
            enddo
         enddo
      enddo
C$OMP END PARALLEL DO

      return
      end
c
