% test matlab mex interfaces to fortran h3dtamatrix local (j) exp evaluator
% Barnett 3/29/12, 2/5/13 for qp3d. 8/5/14 3D random plane waves w/ Jin.

% This test code has three tests; change "if 0" into "if 1" below to include
% each one.

clear;
if 0  % 1) follow same setup as in h3dtamatrix_dr.f: test a couple of pt evals
  k = 10;
  ntarg=10;
  ztarg = [(1:ntarg)/1000; 0.3*ones(1,ntarg); 0.6*ones(1,ntarg)];
  znor = [-1/3*ones(1,ntarg); 1/3+(1:ntarg)/1000; 2/3*ones(1,ntarg)];
  center = [0;0;1];
  rscale = 1;
  nterms = 30;  % p
  n = 5; t = 3; % (t,n) matrix element to test
  A = h3dtamatrix(k,rscale,center,nterms,ztarg); % test values-only call
  [A An] = h3dtamatrix(k,rscale,center,nterms,ztarg,znor);
  A(t,n), An(t,n)  % numbers should agree with h3dtamatrix_dr.f

  % test h3dtaeval interface...
  lpp = (nterms+1)^2; clm = rand(lpp,1) + 1i*rand(lpp,1);
  u = h3dtaeval(k,rscale,center,nterms,clm,ztarg,znor); % test values-only call
  [u un] = h3dtaeval(k,rscale,center,nterms,clm,ztarg,znor);
  disp('should be small:'), norm(u - A*clm)   % should be zero
  norm(un - An*clm)  % "
end

if 0  % 2) evaluate and plot a slice for Re of each local exp basis func...
  k=10; p=4; center = [0;0;0]; rscale = 1.0;
  x=-1:0.03:1; nx = numel(x); pp = (p+1)^2;
  [xx yy] = meshgrid(x);  M = numel(xx);
  %ztarg=[xx(:)';yy(:)';0.3+0*xx(:)'];   % slice plane z=.3
  ztarg=[xx(:)';0*xx(:)';yy(:)'];       % slice y=0 plane
  [A] = h3dtamatrix(k,rscale,center,p,ztarg,ztarg);  % znor is a dummy here
  figure; for j=1:pp, subplot(p+1,p+1,j);
    imagesc(x,x,reshape(real(A(:,j)),[nx nx])); axis equal; caxis(.3*[-1 1]);
    set(gca,'ydir','normal'); end
  % test n=m=0 case against known functions...
  z = k*sqrt(sum((ztarg-repmat(center,[1 M])).^2,1));
  a0=sin(z)./z; %   sqrt(pi./(2*z)).*besselj(1/2,z);
  norm(a0-A(:,1).') / norm(a0)  % agrees to 1e-15
  a1=sqrt(3)*(sin(z)./z.^2 - cos(z)./z) .* (k*ztarg(3,:)./z); %n=1,m=0
  norm(a1-A(:,3).') / norm(a1) % agrees to 1e-15
end

if 1  % 3) plane wave or random plane wave generation:
  expt = 'p';   % 'p' = z plane wave error check, 'r' = random plane wave
  p = 180;  % nterms: check 'p' expt converges over domain, use same for 'r'
  lpp = (p+1)^2; clm = zeros(lpp,1);  % coeffs
  if expt=='p' % Jacobi-Anger for z plane wave:
    i=1; for n=0:p, clm(i)=1i^n * sqrt(2*n+1); i=i+2*(n+1); end % m=0 indices i
  else % Random plane waves in 3D:
    i=1; for n=0:p, clm(i-n:i+n)=(randn(2*n+1,1)+1i*randn(2*n+1,1));
      %/ sqrt(2*n+1);  % factor seems wrong.
      i=i+2*(n+1); end % m=0 indices i
  end
  k = 100; rscale = 1;   % rscale = 1 seems to give no scaling
  center = [0;0;0];   % Mk = 100
  x=-1:0.01:1; nx = numel(x); pp = (p+1)^2;
  [xx yy] = meshgrid(x);  M = numel(xx);
  ztarg=[xx(:)'; 0.1 + 0*xx(:)';yy(:)'];       % slice in y=0.1 plane
  tic, u = h3dtaeval(k,rscale,center,p,clm,ztarg,ztarg); % eval u: znor is dummy
  toc; figure; if expt=='p'; subplot(1,2,1); end
  imagesc(x,x,reshape(real(u),[nx nx])); axis xy equal tight; caxis(5*[-1 1]);
  if expt=='p', title('Jacobi-Anger sum for e^{ikz}');
    subplot(1,2,2); imagesc(x,x,log10(abs(reshape(u,[nx nx])-exp(1i*k*yy))));
  axis xy equal tight; caxis([-16 0]); title('log_{10} error'); colorbar, end
end
